@@ -97,10 +97,14 @@ get_default_route(void)
 		char buffer[1024]; 
 		unsigned int ip, gw, flags, ref, use, metric, mask, mtu, window, irtt;
 		int retval;
+		char *rv;
 		
-		fgets(buffer, 1024, fp);
+		rv = fgets(buffer, 1024, fp);
+		if (!rv) {
+			break;
+		}
 		
-		retval = sscanf(buffer, "%49s %x %x %x %d %d %d %x %d %d %d",
+		retval = sscanf(buffer, "%49s %x %x %x %u %u %u %x %u %u %u",
 				device, &ip, &gw, &flags, &ref, &use, &metric, &mask, &mtu, &window, &irtt);
 		
 		if (retval != 11) continue;
@@ -281,6 +285,7 @@ void
 get_wireless_info (DevInfo *devinfo)
 {
 	int fd;
+	int newqual;
 	wireless_info info = {0};
 
 	fd = iw_sockets_open ();
@@ -303,9 +308,16 @@ get_wireless_info (DevInfo *devinfo)
 		info.has_stats = 1;
 
 	if (info.has_stats) {
-		if (devinfo->qual != info.stats.qual.qual) {
-			devinfo->qual = info.stats.qual.qual;
+		if ((iw_get_range_info(fd, devinfo->name, &info.range) >= 0) && (info.range.max_qual.qual > 0)) {
+			newqual = 0.5f + (100.0f * info.stats.qual.qual) / (1.0f * info.range.max_qual.qual);
+		} else {
+			newqual = info.stats.qual.qual;
 		}
+
+		newqual = CLAMP(newqual, 0, 100);
+		if (devinfo->qual != newqual)
+			devinfo->qual = newqual;
+
 	} else {
 		devinfo->qual = 0;
 	}
