@@ -30,6 +30,12 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
+#include <libmate-desktop/mate-aboutdialog.h>
+#include <libmate-desktop/mate-colorbutton.h>
+
+#define MATE_DESKTOP_USE_UNSTABLE_API
+#include <libmate-desktop/mate-desktop-utils.h>
+
 #include "backend.h"
 
  /* Icons for the interfaces */
@@ -124,7 +130,8 @@ open_uri (GtkWidget *parent, const char *url, GError **error)
 
 	screen = gtk_widget_get_screen (parent);
 	cmdline = g_strconcat ("xdg-open ", url, NULL);
-	ret = gdk_spawn_command_line_on_screen (screen, cmdline, error);
+
+	ret = mate_gdk_spawn_command_line_on_screen (screen, cmdline, error);
 	g_free (cmdline);
 
 	return ret;
@@ -165,12 +172,12 @@ applet_change_size_or_orient(MatePanelApplet *applet_widget, int arg1, MateNetsp
 	size = mate_panel_applet_get_size(applet_widget);
 	orient = mate_panel_applet_get_orient(applet_widget);
 	
-	gtk_widget_ref(applet->pix_box);
-	gtk_widget_ref(applet->in_pix);
-	gtk_widget_ref(applet->in_label);
-	gtk_widget_ref(applet->out_pix);
-	gtk_widget_ref(applet->out_label);
-	gtk_widget_ref(applet->sum_label);
+	g_object_ref(applet->pix_box);
+	g_object_ref(applet->in_pix);
+	g_object_ref(applet->in_label);
+	g_object_ref(applet->out_pix);
+	g_object_ref(applet->out_label);
+	g_object_ref(applet->sum_label);
 
 	if (applet->in_box) {
 		gtk_container_remove(GTK_CONTAINER(applet->in_box), applet->in_label);
@@ -224,12 +231,12 @@ applet_change_size_or_orient(MatePanelApplet *applet_widget, int arg1, MateNetsp
 	gtk_box_pack_start(GTK_BOX(applet->sum_box), applet->sum_label, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(applet->box), applet->pix_box, FALSE, FALSE, 0);
 	
-	gtk_widget_unref(applet->pix_box);
-	gtk_widget_unref(applet->in_pix);
-	gtk_widget_unref(applet->in_label);
-	gtk_widget_unref(applet->out_pix);
-	gtk_widget_unref(applet->out_label);
-	gtk_widget_unref(applet->sum_label);
+	g_object_unref(applet->pix_box);
+	g_object_unref(applet->in_pix);
+	g_object_unref(applet->in_label);
+	g_object_unref(applet->out_pix);
+	g_object_unref(applet->out_label);
+	g_object_unref(applet->sum_label);
 
 	if (applet->show_sum) {
 		gtk_box_pack_start(GTK_BOX(applet->box), applet->sum_box, TRUE, TRUE, 0);
@@ -245,6 +252,7 @@ applet_change_size_or_orient(MatePanelApplet *applet_widget, int arg1, MateNetsp
 	gtk_container_add(GTK_CONTAINER(applet->applet), applet->box);
 }
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 /* Change the background of the applet according to
  * the panel background.
  */
@@ -278,6 +286,7 @@ change_background_cb(MatePanelApplet *applet_widget,
 			break;
 	}
 }
+#endif
 
 
 /* Change the icons according to the selected device
@@ -316,8 +325,8 @@ change_icons(MateNetspeedApplet *applet)
 
 	gtk_image_set_from_pixbuf(GTK_IMAGE(applet->out_pix), out_arrow);
 	gtk_image_set_from_pixbuf(GTK_IMAGE(applet->in_pix), in_arrow);
-	gdk_pixbuf_unref(in_arrow);
-	gdk_pixbuf_unref(out_arrow);
+	g_object_unref(in_arrow);
+	g_object_unref(out_arrow);
 	
 	if (applet->devinfo.running) {
 		gtk_widget_show(applet->in_box);
@@ -352,9 +361,8 @@ update_quality_icon(MateNetspeedApplet *applet)
 	unsigned int q;
 	
 	q = (applet->devinfo.qual);
-	q = logf (q / 3.0f) + 0.25f;
-
-	g_assert(q >= 0 && q < 4);
+	q /= 25;
+	q = CLAMP(q, 0, 3); /* q out of range would crash when accessing qual_pixbufs[q] */
 	gtk_image_set_from_pixbuf (GTK_IMAGE(applet->qual_pix), applet->qual_pixbufs[q]);
 }
 
@@ -386,8 +394,11 @@ init_quality_pixbufs(MateNetspeedApplet *applet)
 static void
 icon_theme_changed_cb(GtkIconTheme *icon_theme, gpointer user_data)
 {
+    MateNetspeedApplet *applet = (MateNetspeedApplet*)user_data;
+
     init_quality_pixbufs(user_data);
-    update_quality_icon(user_data);
+    if (applet->devinfo.type == DEV_WIRELESS && applet->devinfo.up)
+        update_quality_icon(user_data);
     change_icons(user_data);
 }    
 
@@ -458,12 +469,11 @@ bytes_to_string(double bytes, gboolean per_sec, gboolean bits, gboolean shortene
  * Some really black magic is going on in here ;-)
  */
 static void
-redraw_graph(MateNetspeedApplet *applet)
+redraw_graph(MateNetspeedApplet *applet, cairo_t *cr)
 {
-	GdkGC *gc;
-	GdkColor color;
 	GtkWidget *da = GTK_WIDGET(applet->drawingarea);
-	GdkWindow *window, *real_window = da->window;
+	GtkStyle *style = gtk_widget_get_style (da);
+	GdkWindow *real_window = gtk_widget_get_window (da);
 	GdkRectangle ra;
 	GtkStateType state;
 	GdkPoint in_points[GRAPH_VALUES], out_points[GRAPH_VALUES];
@@ -472,50 +482,64 @@ redraw_graph(MateNetspeedApplet *applet)
 	char *text; 
 	int i, offset, w, h;
 	double max_val;
+	double dash[2] = { 1.0, 2.0 };
 	
-	gc = gdk_gc_new (real_window);
-	gdk_drawable_get_size(real_window, &w, &h);
+	w = gdk_window_get_width (real_window);
+	h = gdk_window_get_height (real_window);
 
-	/* use doublebuffering to avoid flickering */
-	window = gdk_pixmap_new(real_window, w, h, -1);
-	
 	/* the graph hight should be: hight/2 <= applet->max_graph < hight */
 	for (max_val = 1; max_val < applet->max_graph; max_val *= 2) ;
 	
 	/* calculate the polygons (GdkPoint[]) for the graphs */ 
 	offset = 0;
-	for (i = applet->index_graph + 1; applet->in_graph[i] < 0; i = (i + 1) % GRAPH_VALUES)
+	for (i = (applet->index_graph + 1) % GRAPH_VALUES; applet->in_graph[i] < 0; i = (i + 1) % GRAPH_VALUES)
 		offset++;
 	for (i = offset + 1; i < GRAPH_VALUES; i++)
 	{
 		int index = (applet->index_graph + i) % GRAPH_VALUES;
-		out_points[i].x = in_points[i].x = ((w - 8) * i) / GRAPH_VALUES + 2;
+		out_points[i].x = in_points[i].x = ((w - 6) * i) / GRAPH_VALUES + 4;
 		in_points[i].y = h - 6 - (int)((h - 8) * applet->in_graph[index] / max_val);
 		out_points[i].y = h - 6 - (int)((h - 8) * applet->out_graph[index] / max_val);
 	}	
-	in_points[offset].x = out_points[offset].x = ((w - 8) * offset) / GRAPH_VALUES + 2;
-	in_points[offset].y = in_points[offset + 1].y;
-	out_points[offset].y = out_points[offset + 1].y;
+	in_points[offset].x = out_points[offset].x = ((w - 6) * offset) / GRAPH_VALUES + 4;
+	in_points[offset].y = in_points[(offset + 1) % GRAPH_VALUES].y;
+	out_points[offset].y = out_points[(offset + 1) % GRAPH_VALUES].y;
 	
 	/* draw the background */
-	gdk_gc_set_rgb_fg_color (gc, &da->style->black);
-	gdk_draw_rectangle (window, gc, TRUE, 0, 0, w, h);
-	
-	color.red = 0x3a00; color.green = 0x8000; color.blue = 0x1400;
-	gdk_gc_set_rgb_fg_color(gc, &color);
-	gdk_draw_rectangle (window, gc, FALSE, 2, 2, w - 6, h - 6);
-	
+	cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
+	cairo_rectangle (cr, 02, 2, w - 6, h - 6);
+	cairo_fill (cr);
+
+	cairo_set_line_width(cr, 1.0);
+	cairo_set_dash (cr, dash, 2, 0);
+
+	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+	cairo_rectangle (cr, 2, 2, w - 6, h - 6);
+	cairo_stroke (cr);
+
 	for (i = 0; i < GRAPH_LINES; i++) {
 		int y = 2 + ((h - 6) * i) / GRAPH_LINES; 
-		gdk_draw_line(window, gc, 2, y, w - 4, y);
+		cairo_move_to (cr, 2, y);
+		cairo_line_to (cr, w - 4, y);
 	}
+	cairo_stroke (cr);
 	
 	/* draw the polygons */
-	gdk_gc_set_line_attributes(gc, 2, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
-	gdk_gc_set_rgb_fg_color(gc, &applet->in_color);
-	gdk_draw_lines(window, gc, in_points + offset, GRAPH_VALUES - offset);
-	gdk_gc_set_rgb_fg_color(gc, &applet->out_color);
-	gdk_draw_lines(window, gc, out_points + offset, GRAPH_VALUES - offset);
+	cairo_set_dash (cr, dash, 0, 1);
+	cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
+	cairo_set_line_join (cr, CAIRO_LINE_JOIN_ROUND);
+
+	gdk_cairo_set_source_color (cr, &applet->in_color);
+	for (i = offset; i < GRAPH_VALUES; i++) {
+		cairo_line_to (cr, in_points[i].x, in_points[i].y);
+	}
+	cairo_stroke (cr);
+
+	gdk_cairo_set_source_color (cr, &applet->out_color);
+	for (i = offset; i < GRAPH_VALUES; i++) {
+		cairo_line_to (cr, out_points[i].x, out_points[i].y);
+	}
+	cairo_stroke (cr);
 
 	/* draw the 2 labels */
 	state = GTK_STATE_NORMAL;
@@ -523,30 +547,31 @@ redraw_graph(MateNetspeedApplet *applet)
 	ra.width = w; ra.height = h;
 	
 	text = bytes_to_string(max_val, TRUE, applet->show_bits, applet->short_unit);
-	add_markup_fgcolor(&text, "white");
+	add_markup_fgcolor(&text, "black");
 	layout = gtk_widget_create_pango_layout (da, NULL);
 	pango_layout_set_markup(layout, text, -1);
 	g_free (text);
-	gtk_paint_layout(da->style, window, state,	FALSE, &ra, da, "max_graph", 3, 2, layout);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_paint_layout(style, cr, state, FALSE, da, "max_graph", 3, 2, layout);
+#else
+	gtk_paint_layout(style, real_window, state, FALSE, &ra, da, "max_graph", 3, 2, layout);
+#endif
 	g_object_unref(G_OBJECT(layout));
 
 	text = bytes_to_string(0.0, TRUE, applet->show_bits, applet->short_unit);
-	add_markup_fgcolor(&text, "white");
+	add_markup_fgcolor(&text, "black");
 	layout = gtk_widget_create_pango_layout (da, NULL);
 	pango_layout_set_markup(layout, text, -1);
 	pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 	g_free (text);
-	gtk_paint_layout(da->style, window, state,	FALSE, &ra, da, "max_graph", 3, h - 4 - logical_rect.height, layout);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_paint_layout(style, cr, state, FALSE, da, "max_graph", 3, h - 4 - logical_rect.height, layout);
+#else
+	gtk_paint_layout(style, real_window, state, FALSE, &ra, da, "max_graph", 3, h - 4 - logical_rect.height, layout);
+#endif
 	g_object_unref(G_OBJECT(layout));
-
-	/* draw the pixmap to the real window */	
-	gdk_draw_drawable(real_window, gc, window, 0, 0, 0, 0, w, h);
-	
-	g_object_unref(G_OBJECT(gc));
-	g_object_unref(G_OBJECT(window));
 }
 
-
 static gboolean
 set_applet_devinfo(MateNetspeedApplet* applet, const char* iface)
 {
@@ -704,7 +729,7 @@ update_applet(MateNetspeedApplet *applet)
 	}
 	/* Redraw the graph of the Infodialog */
 	if (applet->drawingarea)
-		redraw_graph(applet);
+		gtk_widget_queue_draw (GTK_WIDGET (applet->drawingarea));
 	
 	/* Save old values... */
 	applet->in_old[applet->index_old] = applet->devinfo.rx;
@@ -793,45 +818,6 @@ help_cb (GtkAction *action, MateNetspeedApplet *ap)
 	display_help (GTK_WIDGET (ap->applet), NULL);
 }
 
-enum {
-	LINK_TYPE_EMAIL,
-	LINK_TYPE_URL
-};
-
-/* handle the links of the about dialog */
-static void
-handle_links (GtkAboutDialog *about, const gchar *link, gpointer data)
-{
-	gchar *new_link;
-	GError *error = NULL;
-	gboolean ret;
-	GtkWidget *dialog;
-
-	switch (GPOINTER_TO_INT (data)){
-	case LINK_TYPE_EMAIL:
-		new_link = g_strdup_printf ("mailto:%s", link);
-		break;
-	case LINK_TYPE_URL:
-		new_link = g_strdup (link);
-		break;
-	default:
-		g_assert_not_reached ();
-	}
-
-	ret = open_uri (GTK_WIDGET (about), new_link, &error);
-
-	if (ret == FALSE) {
-    		dialog = gtk_message_dialog_new (GTK_WINDOW (dialog), 
-						 GTK_DIALOG_DESTROY_WITH_PARENT, 
-						 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-				                 _("Failed to show:\n%s"), new_link); 
-    		gtk_dialog_run (GTK_DIALOG (dialog));
-    		gtk_widget_destroy (dialog);
-		g_error_free(error);
-	}
-	g_free (new_link);
-}
-
 /* Just the about window... If it's already open, just fokus it
  */
 static void
@@ -847,22 +833,15 @@ about_cb(GtkAction *action, gpointer data)
 		"Perberos <perberos@gmail.com>",
 		NULL
 	};
-    
-	gtk_about_dialog_set_email_hook ((GtkAboutDialogActivateLinkFunc) handle_links,
-					 GINT_TO_POINTER (LINK_TYPE_EMAIL), NULL);
 	
-	gtk_about_dialog_set_url_hook ((GtkAboutDialogActivateLinkFunc) handle_links,
-				       GINT_TO_POINTER (LINK_TYPE_URL), NULL);
-	
-	gtk_show_about_dialog (NULL, 
+	mate_show_about_dialog (NULL, 
 			       "version", VERSION, 
-			       "copyright", "Copyright 2002 - 2003 Jörgen Scheibengruber\nCopyright 2011 Stefano Karapetsas",
+			       "copyright", "Copyright 2002 - 2003 Jörgen Scheibengruber\nCopyright 2011-2014 Stefano Karapetsas",
 			       "comments", _("A little applet that displays some information on the traffic on the specified network device"),
 			       "authors", authors, 
 			       "documenters", NULL, 
 			       "translator-credits", _("translator-credits"),
 			       "website", "http://www.mate-desktop.org/",
-			       "website-label", _("MATE Website"),
 			       "logo-icon-name", LOGO_ICON,
 			       NULL);
 	
@@ -994,7 +973,6 @@ settings_cb(GtkAction *action, gpointer data)
 	GtkWidget *show_icon_checkbutton;
 	GtkWidget *change_icon_checkbutton;
 	GtkSizeGroup *category_label_size_group;
-	GtkSizeGroup *category_units_size_group;
   	gchar *header_str;
 	GList *ptr, *devices;
 	int i, active = -1;
@@ -1008,7 +986,6 @@ settings_cb(GtkAction *action, gpointer data)
 	}
 
 	category_label_size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
-	category_units_size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
   
 	applet->settings = GTK_DIALOG(gtk_dialog_new_with_buttons(_("Mate Netspeed Preferences"), 
 								  NULL, 
@@ -1059,15 +1036,27 @@ settings_cb(GtkAction *action, gpointer data)
 	gtk_size_group_add_widget(category_label_size_group, network_device_label);
 	gtk_box_pack_start(GTK_BOX (network_device_hbox), network_device_label, FALSE, FALSE, 0);
 	
+#if GTK_CHECK_VERSION (3, 0, 0)
+	applet->network_device_combo = gtk_combo_box_text_new();
+#else
 	applet->network_device_combo = gtk_combo_box_new_text();
+#endif
 	gtk_label_set_mnemonic_widget(GTK_LABEL(network_device_label), applet->network_device_combo);
 	gtk_box_pack_start (GTK_BOX (network_device_hbox), applet->network_device_combo, TRUE, TRUE, 0);
 
 	/* Default means device with default route set */
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(applet->network_device_combo), _("Default"));
+#else
 	gtk_combo_box_append_text(GTK_COMBO_BOX(applet->network_device_combo), _("Default"));
+#endif
 	ptr = devices = get_available_devices();
 	for (i = 0; ptr; ptr = g_list_next(ptr)) {
+#if GTK_CHECK_VERSION (3, 0, 0)
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(applet->network_device_combo), ptr->data);
+#else
 		gtk_combo_box_append_text(GTK_COMBO_BOX(applet->network_device_combo), ptr->data);
+#endif
 		if (g_str_equal(ptr->data, applet->devinfo.name)) active = (i + 1);
 		++i;
 	}
@@ -1118,29 +1107,42 @@ settings_cb(GtkAction *action, gpointer data)
 	g_signal_connect(G_OBJECT (applet->settings), "response",
 			 G_CALLBACK(pref_response_cb), (gpointer)applet);
 
-	gtk_container_add(GTK_CONTAINER(applet->settings->vbox), vbox); 
+	gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area (applet->settings)), vbox); 
 
 	gtk_widget_show_all(GTK_WIDGET(applet->settings));
 }
 
 static gboolean
+#if GTK_CHECK_VERSION (3, 0, 0)
+da_draw(GtkWidget *widget, cairo_t *cr, gpointer data)
+#else
 da_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data)
+#endif
 {
 	MateNetspeedApplet *applet = (MateNetspeedApplet*)data;
 	
-	redraw_graph(applet);
+#if !GTK_CHECK_VERSION (3, 0, 0)
+	cairo_t *cr;
+	cr = gdk_cairo_create (event->window);
+	gdk_cairo_region (cr, event->region);
+	cairo_clip (cr);
+#endif
+	redraw_graph(applet, cr);
+#if !GTK_CHECK_VERSION (3, 0, 0)
+	cairo_destroy (cr);
+#endif
 	
 	return FALSE;
 }	
 
 static void
-incolor_changed_cb (GtkColorButton *cb, gpointer data)
+incolor_changed_cb (MateColorButton *cb, gpointer data)
 {
 	MateNetspeedApplet *applet = (MateNetspeedApplet*)data;
 	gchar *color;
 	GdkColor clr;
 	
-	gtk_color_button_get_color (cb, &clr);
+	mate_color_button_get_color (cb, &clr);
 	applet->in_color = clr;
 	
 	color = g_strdup_printf ("#%04x%04x%04x", clr.red, clr.green, clr.blue);
@@ -1149,13 +1151,13 @@ incolor_changed_cb (GtkColorButton *cb, gpointer data)
 }
 
 static void
-outcolor_changed_cb (GtkColorButton *cb, gpointer data)
+outcolor_changed_cb (MateColorButton *cb, gpointer data)
 {
 	MateNetspeedApplet *applet = (MateNetspeedApplet*)data;
 	gchar *color;
 	GdkColor clr;
 	
-	gtk_color_button_get_color (cb, &clr);
+	mate_color_button_get_color (cb, &clr);
 	applet->out_color = clr;
 	
 	color = g_strdup_printf ("#%04x%04x%04x", clr.red, clr.green, clr.blue);
@@ -1227,7 +1229,7 @@ showinfo_cb(GtkAction *action, gpointer data)
 	gtk_table_set_col_spacings(GTK_TABLE(table), 15);
 	
 	da_frame = gtk_frame_new(NULL);
-	gtk_frame_set_shadow_type(GTK_FRAME(da_frame), GTK_SHADOW_IN);
+	gtk_frame_set_shadow_type(GTK_FRAME(da_frame), GTK_SHADOW_NONE);
 	applet->drawingarea = GTK_DRAWING_AREA(gtk_drawing_area_new());
 	gtk_widget_set_size_request(GTK_WIDGET(applet->drawingarea), -1, 180);
 	gtk_container_add(GTK_CONTAINER(da_frame), GTK_WIDGET(applet->drawingarea));
@@ -1236,11 +1238,11 @@ showinfo_cb(GtkAction *action, gpointer data)
 	incolor_label = gtk_label_new_with_mnemonic(_("_In graph color"));
 	outcolor_label = gtk_label_new_with_mnemonic(_("_Out graph color"));
 	
-	incolor_sel = gtk_color_button_new ();
-	outcolor_sel = gtk_color_button_new ();
+	incolor_sel =  mate_color_button_new ();
+	outcolor_sel = mate_color_button_new ();
 	
-	gtk_color_button_set_color (GTK_COLOR_BUTTON (incolor_sel),  &applet->in_color);
-	gtk_color_button_set_color (GTK_COLOR_BUTTON (outcolor_sel),  &applet->out_color);
+	mate_color_button_set_color (MATE_COLOR_BUTTON (incolor_sel),  &applet->in_color);
+	mate_color_button_set_color (MATE_COLOR_BUTTON (outcolor_sel),  &applet->out_color);
 
 	gtk_label_set_mnemonic_widget(GTK_LABEL(incolor_label), incolor_sel);
 	gtk_label_set_mnemonic_widget(GTK_LABEL(outcolor_label), outcolor_sel);
@@ -1347,9 +1349,15 @@ showinfo_cb(GtkAction *action, gpointer data)
 		gtk_table_attach_defaults (GTK_TABLE (table), essid_text, 1, 4, 4, 5);
 	}
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	g_signal_connect(G_OBJECT(applet->drawingarea), "draw",
+			 G_CALLBACK(da_draw),
+			 (gpointer)applet);
+#else
 	g_signal_connect(G_OBJECT(applet->drawingarea), "expose_event",
 			 GTK_SIGNAL_FUNC(da_expose_event),
 			 (gpointer)applet);
+#endif
 
 	g_signal_connect(G_OBJECT(incolor_sel), "color_set", 
 			 G_CALLBACK(incolor_changed_cb),
@@ -1366,7 +1374,7 @@ showinfo_cb(GtkAction *action, gpointer data)
 	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 0);
 
-	gtk_container_add(GTK_CONTAINER(applet->details->vbox), box); 
+	gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area (applet->details)), box); 
 	gtk_widget_show_all(GTK_WIDGET(applet->details));
 }
 
@@ -1413,27 +1421,27 @@ applet_button_press(GtkWidget *widget, GdkEventButton *event, MateNetspeedApplet
 		
 		if (applet->up_cmd && applet->down_cmd)
 		{
-			char *question;
+			const char *question;
 			int response;
 			
 			if (applet->devinfo.up) 
 			{
-				question = g_strdup_printf(_("Do you want to disconnect %s now?"), applet->devinfo.name); 
+				question = _("Do you want to disconnect %s now?");
 			} 
 			else
 			{
-				question = g_strdup_printf(_("Do you want to connect %s now?"), applet->devinfo.name);
+				question = _("Do you want to connect %s now?");
 			}
 			
 			applet->connect_dialog = gtk_message_dialog_new(NULL, 
 					GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 					GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
 					"%s",
-					question);
+					question,
+					applet->devinfo.name);
 			response = gtk_dialog_run(GTK_DIALOG(applet->connect_dialog));
 			gtk_widget_destroy (applet->connect_dialog);
 			applet->connect_dialog = NULL;
-			g_free(question);
 			
 			if (response == GTK_RESPONSE_YES)
 			{
@@ -1447,10 +1455,11 @@ applet_button_press(GtkWidget *widget, GdkEventButton *event, MateNetspeedApplet
 				if (!g_spawn_command_line_async(command, &error))
 				{
 					
-					dialog = gtk_message_dialog_new(NULL, 
+					dialog = gtk_message_dialog_new_with_markup(NULL,
 							GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 							GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-							"%s",
+							_("<b>Running command %s failed</b>\n%s"),
+							command,
 							error->message);
 					gtk_dialog_run (GTK_DIALOG (dialog));
 					gtk_widget_destroy (dialog);
@@ -1470,7 +1479,14 @@ applet_button_press(GtkWidget *widget, GdkEventButton *event, MateNetspeedApplet
 static void
 applet_destroy(MatePanelApplet *applet_widget, MateNetspeedApplet *applet)
 {
+	GtkIconTheme *icon_theme;
+
 	g_assert(applet);
+
+	icon_theme = gtk_icon_theme_get_default();
+	g_object_disconnect(G_OBJECT(icon_theme), "changed",
+			    G_CALLBACK(icon_theme_changed_cb), (gpointer)applet,
+			    NULL);
 	
 	g_source_remove(applet->timeout_id);
 	applet->timeout_id = 0;
@@ -1695,7 +1711,11 @@ mate_netspeed_applet_factory(MatePanelApplet *applet_widget, const gchar *iid, g
 	mate_panel_applet_set_flags(applet_widget, MATE_PANEL_APPLET_EXPAND_MINOR);
 	
 	applet->timeout_id = g_timeout_add(applet->refresh_time,
+#if GTK_CHECK_VERSION (3, 0, 0)
+                           (GSourceFunc)timeout_function,
+#else
                            (GtkFunction)timeout_function,
+#endif
                            (gpointer)applet);
 
 	g_signal_connect(G_OBJECT(applet_widget), "change_size",
@@ -1710,10 +1730,12 @@ mate_netspeed_applet_factory(MatePanelApplet *applet_widget, const gchar *iid, g
                            G_CALLBACK(applet_change_size_or_orient),
                            (gpointer)applet);
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	g_signal_connect(G_OBJECT(applet_widget), "change_background",
                            G_CALLBACK(change_background_cb),
 			   (gpointer)applet);
-		       
+#endif
+
 	g_signal_connect(G_OBJECT(applet->in_label), "size_request",
                            G_CALLBACK(label_size_request_cb),
                            (gpointer)applet);
